// The file implements the Entity interface on the ComponentDefinition struct.
// This is not autogenerated.
package component

import (
	"fmt"
	"os"
	"path/filepath"

	"github.com/gofrs/uuid"
	"github.com/meshery/meshkit/database"
	"github.com/meshery/meshkit/models/meshmodel/entity"
	"github.com/meshery/meshkit/utils"
	"gorm.io/gorm/clause"
)

type Styles = ComponentDefinition_Styles

func (c ComponentDefinition) TableName() string {
	return "component_definition_dbs"
}

func (c ComponentDefinition) Type() entity.EntityType {
	return entity.ComponentDefinition
}

func (c *ComponentDefinition) GenerateID() (uuid.UUID, error) {
	return uuid.NewV4()
}

func (c ComponentDefinition) GetID() uuid.UUID {
	return c.Id
}

func (c *ComponentDefinition) GetEntityDetail() string {
	return fmt.Sprintf("type: %s, definition version: %s, name: %s, model: %s, version: %s", c.Type(), c.Version, c.DisplayName, c.Model.Name, c.Model.Version)
}

func (c *ComponentDefinition) Create(db *database.Handler, hostID uuid.UUID) (uuid.UUID, error) {
	c.Id, _ = c.GenerateID()

	isAnnotation := c.Metadata.IsAnnotation

	if c.Component.Schema == "" && !isAnnotation { //For components which has an empty schema and is not an annotation, return error
		// return ErrEmptySchema()
		return uuid.Nil, nil
	}

	mid, err := c.Model.Create(db, hostID)
	if err != nil {
		return uuid.UUID{}, err
	}

	if !utils.IsSchemaEmpty(c.Component.Schema) {
		if c.Metadata.AdditionalProperties == nil {
			c.Metadata.AdditionalProperties = make(map[string]interface{})
		}
		c.Metadata.AdditionalProperties["hasInvalidSchema"] = true
	}

	c.ModelId = mid
	err = db.Omit(clause.Associations).Create(&c).Error
	return c.Id, err
}

func (m *ComponentDefinition) UpdateStatus(db *database.Handler, status entity.EntityStatus) error {
	return nil
}

func (c ComponentDefinition) WriteComponentDefinition(componentDirPath string, fileType string) (bool, error) {
	if c.Component.Kind == "" {
		return false, nil
	}
	componentPath := filepath.Join(componentDirPath, c.Component.Kind+"."+fileType)
	if _, err := os.Stat(componentPath); err != nil {
		if fileType == "yaml" {
			err := utils.WriteYamlToFile[ComponentDefinition](componentPath, c)
			return false, err
		}
		err := utils.WriteJSONToFile[ComponentDefinition](componentPath, c)
		return false, err
	}
	return true, nil
}

func (c *ComponentDefinition) ReplaceSVGData(baseDir string) error {

	compStyle := c.Styles
	if compStyle != nil {
		svgColor, err := utils.ReadSVGData(baseDir, compStyle.SvgColor)
		if err == nil {
			compStyle.SvgColor = svgColor
		} else {
			return err
		}
		svgWhite, err := utils.ReadSVGData(baseDir, compStyle.SvgWhite)
		if err == nil {
			compStyle.SvgWhite = svgWhite
		} else {
			return err
		}
		svgComplete, err := utils.ReadSVGData(baseDir, compStyle.SvgComplete)
		if err == nil {
			compStyle.SvgComplete = svgComplete
		} else {
			return err
		}
	}
	c.Styles = compStyle
	return nil
}
